import { Vector3 } from "three";
import { DataContainer } from "./DataContainer";
import { Node } from "./Node";
/**
 * A cubic octant.
 *
 * @param T - The type of the data.
 */
export declare class CubicOctant<T> implements Node, DataContainer<T> {
    min: Vector3;
    children: Node[];
    data: T;
    /**
     * The size of this octant.
     */
    size: number;
    /**
     * Constructs a new cubic octant.
     *
     * @param min - The lower bounds.
     * @param size - The size of the octant.
     */
    constructor(min?: Vector3, size?: number);
    /**
     * The upper bounds of this octant.
     *
     * Attention: Accessing this property creates a new vector!
     */
    get max(): Vector3;
    /**
     * Computes the center of this octant.
     *
     * @param target - A target vector.
     * @return The center.
     */
    getCenter(target: Vector3): Vector3;
    /**
     * Returns the size of this octant as a vector.
     *
     * @param target - A target vector.
     * @return The size.
     */
    getDimensions(target: Vector3): Vector3;
    /**
     * Splits this octant into eight smaller ones.
     */
    split(): void;
}
